@@ -105,7 +105,7 @@ static void ole_info_unref (MSOleInfo *info);
 /**
  * ole_get_block :
  * @ole: the infile
- * @block:
+ * @block: block number
  * @buffer: optionally NULL
  *
  * Read a block of data from the underlying input.
@@ -140,7 +140,7 @@ ole_get_block (GsfInfileMSOle const *ole, guint32 block, guint8 *buffer)
  * table for the list starting in @block.
  *
  * Returns: TRUE on error.
- */
+ **/
 static gboolean
 ole_make_bat (MSOleBAT const *metabat, size_t size_guess, guint32 block,
 	      MSOleBAT *res)
@@ -155,7 +155,8 @@ ole_make_bat (MSOleBAT const *metabat, size_t size_guess, guint32 block,
 	if (block < metabat->num_blocks)
 		do {
 			/* Catch cycles in the bat list */
-			g_return_val_if_fail (0 == (used[block/8] & (1 << (block & 0x7))), TRUE);
+			if (used[block/8] & (1 << (block & 0x7)))
+				break;
 			used[block/8] |= 1 << (block & 0x7);
 
 			g_array_append_val (bat, block);
@@ -187,16 +188,9 @@ ols_bat_release (MSOleBAT *bat)
 	}
 }
 
-/**
- * ole_info_read_metabat :
- * @ole:
- * @bats:
- *
- * A small utility routine to read a set of references to bat blocks
+/* A small utility routine to read a set of references to bat blocks
  * either from the OLE header, or a meta-bat block.
- *
- * Returns: a pointer to the element after the last position filled.
- **/
+ * Returns: a pointer to the element after the last position filled. */
 static guint32 *
 ole_info_read_metabat (GsfInfileMSOle *ole, guint32 *bats, guint32 max_bat,
 		       guint32 const *metabat, guint32 const *metabat_end)
@@ -227,14 +221,7 @@ ole_info_read_metabat (GsfInfileMSOle *ole, guint32 *bats, guint32 max_bat,
 	return bats;
 }
 
-/**
- * gsf_ole_get_guint32s :
- * @dst:
- * @src:
- * @num_bytes:
- *
- * Copy some some raw data into an array of guint32.
- **/
+/* Copy some some raw data into an array of guint32. */
 static void
 gsf_ole_get_guint32s (guint32 *dst, guint8 const *src, int num_bytes)
 {
@@ -286,18 +273,11 @@ ole_dirent_cmp (MSOleDirent const *a, MSOleDirent const *b)
 	return strcmp (b->collation_name, a->collation_name);
 }
 
-/**
- * ole_dirent_new :
- * @ole:
- * @entry:
- * @parent: optional
- *
- * Parse dirent number @entry and recursively handle its siblings and children.
- *
- * Returns: The dirent
- **/
+/* Parse dirent number @entry and recursively handle its siblings and children.
+ * parent is optional. */
 static MSOleDirent *
-ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent)
+ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent,
+		guint8 *seen_before)
 {
 	MSOleDirent *dirent;
 	guint32 block, next, prev, child, size;
@@ -308,9 +288,14 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent)
 	if (entry >= DIRENT_MAGIC_END)
 		return NULL;
 
-	block = OLE_BIG_BLOCK (entry * DIRENT_SIZE, ole);
+	g_return_val_if_fail (entry <= G_MAXUINT / DIRENT_SIZE, NULL);
 
+	block = OLE_BIG_BLOCK (entry * DIRENT_SIZE, ole);
 	g_return_val_if_fail (block < ole->bat.num_blocks, NULL);
+
+	g_return_val_if_fail (!seen_before[entry], NULL);
+	seen_before[entry] = TRUE;
+
 	data = ole_get_block (ole, ole->bat.block [block], NULL);
 	if (data == NULL)
 		return NULL;
@@ -386,17 +371,11 @@ ole_dirent_new (GsfInfileMSOle *ole, guint32 entry, MSOleDirent *parent)
 			dirent, (GCompareFunc)ole_dirent_cmp);
 
 	/* NOTE : These links are a tree, not a linked list */
-	if (prev == entry) {
-		g_warning ("Invalid OLE file with a cycle in its directory tree");
-	} else
-		ole_dirent_new (ole, prev, parent); 
-	if (next == entry) {
-		g_warning ("Invalid OLE file with a cycle in its directory tree");
-	} else
-		ole_dirent_new (ole, next, parent); 
+	ole_dirent_new (ole, prev, parent, seen_before); 
+	ole_dirent_new (ole, next, parent, seen_before); 
 
 	if (dirent->is_directory)
-		ole_dirent_new (ole, child, dirent);
+		ole_dirent_new (ole, child, dirent, seen_before);
 	else if (child != DIRENT_MAGIC_END)
 		g_warning ("A non directory stream with children ?");
 
@@ -446,15 +425,8 @@ ole_info_ref (MSOleInfo *info)
 	return info;
 }
 
-/**
- * ole_dup:
- * @src:
- *
- * Utility routine to _partially_ replicate a file.  It does NOT copy the bat
- * blocks, or init the dirent.
- *
- * Returns: the partial duplicate.
- **/
+/* Utility routine to _partially_ replicate a file.  It does NOT copy the bat
+ * blocks, or init the dirent. */
 static GsfInfileMSOle *
 ole_dup (GsfInfileMSOle const *src, GError **err)
 {
@@ -472,29 +444,24 @@ ole_dup (GsfInfileMSOle const *src, GError **err)
 	}
 
 	dst = (GsfInfileMSOle *)g_object_new (GSF_INFILE_MSOLE_TYPE, NULL);
+	if (G_UNLIKELY (NULL == dst)) return NULL;
+
 	dst->input = input;
 	dst->info  = ole_info_ref (src->info);
-
 	/* buf and buf_size are initialized to NULL */
 
 	return dst;
 }
 
-/**
- * ole_init_info :
- * @ole:
- * @err: optionally NULL
- *
- * Read an OLE header and do some sanity checking
+/* Read an OLE header and do some sanity checking
  * along the way.
- *
- * Returns: TRUE on error setting @err if it is supplied.
- **/
+ * Returns: %TRUE on error setting @err if it is supplied. */
 static gboolean
 ole_init_info (GsfInfileMSOle *ole, GError **err)
 {
 	static guint8 const signature[] =
 		{ 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1 };
+	guint8 *seen_before;
 	guint8 const *header, *tmp;
 	guint32 *metabat = NULL;
 	MSOleInfo *info;
@@ -555,7 +522,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	}
 
 	/* very rough heuristic, just in case */
-	if (num_bat < info->max_block) {
+	if (num_bat < info->max_block && info->num_sbat < info->max_block) {
 		info->bb.bat.num_blocks = num_bat * (info->bb.size / BAT_INDEX_SIZE);
 		info->bb.bat.block	= g_new0 (guint32, info->bb.bat.num_blocks);
 
@@ -636,7 +603,10 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	}
 
 	/* Read the directory */
-	ole->dirent = info->root_dir = ole_dirent_new (ole, 0, NULL);
+	seen_before = g_malloc0 ((ole->bat.num_blocks << info->bb.shift) * DIRENT_SIZE + 1);
+	ole->dirent = info->root_dir =
+		ole_dirent_new (ole, 0, NULL, seen_before);
+	g_free (seen_before);
 	if (ole->dirent == NULL) {
 		if (err != NULL)
 			*err = g_error_new (gsf_input_error_id (), 0,
@@ -936,8 +906,8 @@ GSF_CLASS (GsfInfileMSOle, gsf_infile_msole,
 
 /**
  * gsf_infile_msole_new :
- * @source:
- * @err:
+ * @source: #GsfInput
+ * @err: optional place to store an error
  *
  * Opens the root directory of an MS OLE file.
  * <note>This adds a reference to @source.</note>
@@ -953,6 +923,8 @@ gsf_infile_msole_new (GsfInput *source, GError **err)
 	g_return_val_if_fail (GSF_IS_INPUT (source), NULL);
 
 	ole = (GsfInfileMSOle *)g_object_new (GSF_INFILE_MSOLE_TYPE, NULL);
+	if (G_UNLIKELY (NULL == ole)) return NULL;
+
 	ole->input = gsf_input_proxy_new (source);
 	gsf_input_set_size (GSF_INPUT (ole), 0);
 
