@@ -80,7 +80,9 @@
 #endif
 
 
+#ifndef HAVE_G_BASE64_ENCODE
 static void base64_init (void);
+#endif
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -131,7 +133,9 @@ gsf_init (void)
 #endif
 
 	g_type_init ();
+#ifndef HAVE_G_BASE64_ENCODE
 	base64_init ();
+#endif
 
 #ifdef _GSF_GTYPE_THREADING_FIXED
 	if (NULL == static_type_module) {
@@ -143,6 +147,21 @@ gsf_init (void)
 #else
 	gsf_init_dynamic (NULL);
 #endif
+
+	{
+		/* Little-endian representation of M_PI.  */
+		static guint8 pibytes[8] = {
+			0x18, 0x2d, 0x44, 0x54, 0xfb, 0x21, 0x09, 0x40
+		};
+
+		/*
+		 * If this fails, see
+		 *   http://bugzilla.gnome.org/show_bug.cgi?id=350973
+		 */
+		double pi = gsf_le_get_double (pibytes);
+		if (!(pi > 3.14 && pi < 3.15))
+			g_error ("Compilation trouble with endianess.");
+	}
 }
 
 /**
@@ -593,6 +612,7 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
 	return result;
 }
 
+#ifndef HAVE_G_BASE64_ENCODE
 /***************************************************************************/
 /* some code taken from evolution/camel/camel-mime-utils.c */
 
@@ -603,7 +623,7 @@ gsf_filename_to_utf8 (char const *filename, gboolean quoted)
  *           Jeffrey Stedfast <fejj@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -638,6 +658,7 @@ base64_init(void)
 	}
 	camel_mime_base64_rank['='] = 0;
 }
+#endif
 
 /**
  * gsf_base64_encode_close :
@@ -651,14 +672,20 @@ base64_init(void)
  * This funcion should be called to when finished encoding everything, to
  * flush off the last little bit.
  *
- * Returns:
- */
+ * Returns: a count of the number of bytes in the final block.
+ **/
 size_t
 gsf_base64_encode_close (guint8 const *in, size_t inlen,
 			 gboolean break_lines, guint8 *out, int *state, unsigned int *save)
 {
-	int c1, c2;
 	guint8 *outptr = out;
+#ifdef HAVE_G_BASE64_ENCODE
+	if (inlen > 0)
+		outptr += gsf_base64_encode_step (in, inlen, break_lines,
+						  outptr, state, save);
+	outptr += g_base64_encode_close (break_lines, outptr, state, save);
+#else
+	int c1, c2;
 
 	if (inlen>0)
 		outptr += gsf_base64_encode_step(in, inlen, break_lines, outptr, state, save);
@@ -692,6 +719,7 @@ gsf_base64_encode_close (guint8 const *in, size_t inlen,
 	*save = 0;
 	*state = 0;
 
+#endif
 	return outptr-out;
 }
 
@@ -714,6 +742,9 @@ size_t
 gsf_base64_encode_step (guint8 const *in, size_t len,
 			gboolean break_lines, guint8 *out, int *state, unsigned int *save)
 {
+#ifdef HAVE_G_BASE64_ENCODE
+	return g_base64_encode_step (in, len, break_lines, out, state, save);
+#else
 	register guint8 const *inptr;
 	register guint8 *outptr;
 
@@ -785,6 +816,7 @@ gsf_base64_encode_step (guint8 const *in, size_t len,
 		 (int)((char *)save)[2]));
 
 	return outptr-out;
+#endif
 }
 
 
@@ -804,6 +836,9 @@ size_t
 gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
 			int *state, guint *save)
 {
+#ifdef HAVE_G_BASE64_ENCODE
+	return g_base64_decode_step (in, len, out, state, save);
+#else
 	register guint8 const *inptr;
 	register guint8 *outptr, c;
 	register unsigned int v;
@@ -848,6 +883,7 @@ gsf_base64_decode_step (guint8 const *in, size_t len, guint8 *out,
 
 	/* if i!= 0 then there is a truncation error! */
 	return outptr-out;
+#endif
 }
 
 /**
@@ -863,17 +899,17 @@ guint8 *
 gsf_base64_encode_simple (guint8 const *data, size_t len)
 {
 	guint8 *out;
-	int state = 0, outlen;
-	unsigned int save = 0;
-	gboolean break_lines = TRUE;
+	int state = 0;
+	guint save = 0;
+	gboolean break_lines = TRUE;  /* This differs from g_base64_encode */
+	size_t outlen = len * 4 / 3 + 5;
 
-	outlen = len * 4 / 3 + 5;
 	if (break_lines)
-		outlen += outlen / BASE64_LINE_LEN + 1;
+		outlen += outlen / 72 + 1;
 	out = g_new (guint8, outlen);
 	outlen = gsf_base64_encode_close (data, len, break_lines,
 					  out, &state, &save);
-	out [outlen] = '\0';
+	out[outlen] = '\0';
 	return out;
 }
 
@@ -890,7 +926,7 @@ size_t
 gsf_base64_decode_simple (guint8 *data, size_t len)
 {
 	int state = 0;
-	unsigned int save = 0;
+	guint save = 0;
 	return gsf_base64_decode_step (data, len, data, &state, &save);
 }
 
